%% Return matrix of centroid coordinates (col 1, x; col 2, y) from binarization and connected-component labeling 
% thrs, double
% for more detail: http://blogs.mathworks.com/steve/2007/08/31/intensity-weighted-centroids/
function spotcoord = centroids(img,thrs)

    img = (img-min(min(img)))/(max(max(img))-min(min(img)));
    img = im2bw(img,thrs);
    spotlist = regionprops(img, 'PixelIdxList', 'PixelList'); % structure array of connected components (groups of pixels) with pixel indices and coordinates as fields
    spotcoord = zeros(length(spotlist),2);
    for k = 1:length(spotlist)
        indices = spotlist(k).PixelIdxList;
        totalI = sum(img(indices)); % total intensity of connected component / region
        xcoord = spotlist(k).PixelList(:, 1);
        ycoord = spotlist(k).PixelList(:, 2);
        spotcoord(k,1) = sum(xcoord.*img(indices))/totalI; % intensity-weighted mean along x
        spotcoord(k,2) = sum(ycoord.*img(indices))/totalI;
    end

end