% 161003 Evolve, TetraSpeck calibration
% upper left pixel in green channel = (5,17)
% upper left pixel in red channel = (275,15)

% 161129 Evolve Delta, TetraSpeck calibration
% upper left pixel in green channel = (10,2)
% upper left pixel in red channel = (273,2)

% 161201 Evolve, TetraSpeck calibration   <<< current
% upper left pixel in green channel = (12,5)
% upper left pixel in red channel = (276,6)

function colocal_check()

    % upper left corner of green and red channels
    green_x = 12-1;
    green_y = 5-1;
    red_x = 276-1;
    red_y = 6-1;

    % input and output
    rootname = 'motor';
    warning('off','MATLAB:imagesci:tiffmexutils:libtiffWarning');

    % extract header and last step from fitdetail
    deetfile = '161206_motor_smallcircle_pif_custom_fitdetail.csv';
    fitdetail = csvread(deetfile);
    header_ind = find(sum(fitdetail~=0,2)>2); % indices of rows with more than two non-zero elements
    lvlitems = find(sum(fitdetail~=0,2)==2); % indices of rows with two non-zero elements
    lvlitems(diff(lvlitems(:,1))==1)=[];
    lvlitems = lvlitems - 1; % second last line for last bleaching event

    for k = 1:size(header_ind,1)
    % for k = 17:17
        pifparam = fitdetail(header_ind(k),:);

        filename = [rootname,'_',num2str(pifparam(1)),'_MMImages.ome.tif'];
        
        
        % read tiff stack using LibTIFF routines
        info = imfinfo(filename);
        nframes = numel(info);
        tiff_obj = Tiff(filename,'r');
        tiff_stack = zeros(512,512,nframes);
        for l = 1:nframes
            tiff_obj.setDirectory(l);
            tiff_stack(:,:,l) = tiff_obj.read();
        end
        tiff_obj.close();

        % extract protein and dna tiff stacks
        xcoord = pifparam(2);
        ycoord = pifparam(3);
        prot_stack = tiff_stack((green_y+ycoord-3):(green_y+ycoord+3),(green_x+xcoord-3):(green_x+xcoord+3),1:1500);
        dna_stack = tiff_stack((red_y+ycoord-3):(red_y+ycoord+3),(red_x+xcoord-3):(red_x+xcoord+3),1:1500);


        % frame_zero's
        [foo, prot_frame_zero] = max(diff(ckfilter(transpose(squeeze(sum(sum(prot_stack,1),2))),[20,50],10,10))); % sharpest rise; laser on
        prot_frame_zero = min([prot_frame_zero, 200]);
        [foo, dna_frame_zero] = max(diff(ckfilter(transpose(squeeze(sum(sum(dna_stack,1),2))),[20,50],10,10)));
        dna_frame_zero = min([dna_frame_zero, 50]);

        prot_img = sum(prot_stack(:,:,(prot_frame_zero+1):fitdetail(header_ind(k)+1,1)),3)/(fitdetail(header_ind(k)+1,1)-prot_frame_zero); % first step
        % prot_img = sum(prot_stack(:,:,(prot_frame_zero+1):fitdetail(lvlitems(k),1)),3)/(fitdetail(lvlitems(k),1)-prot_frame_zero); % all steps
        dna_img = sum(dna_stack(:,:,(dna_frame_zero+1):900),3)/(900-dna_frame_zero);

        prot_img_spot = prot_img;
        dna_img_spot = dna_img;
        [xgrid,ygrid] = meshgrid(1:7);
        spot_mask = sqrt((xgrid-4).^2+(ygrid-4).^2)<=3;
        prot_img_spot(not(spot_mask)) = 0;
        dna_img_spot(not(spot_mask)) = 0;

        prot_x = sum((1:7).*sum(prot_img_spot,1))/sum(sum(prot_img_spot)) - 0.5;
        prot_y = sum((1:7).*transpose(sum(prot_img_spot,2)))/sum(sum(prot_img_spot)) - 0.5;
        dna_x = sum((1:7).*sum(dna_img_spot,1))/sum(sum(dna_img_spot)) - 0.5;
        dna_y = sum((1:7).*transpose(sum(dna_img_spot,2)))/sum(sum(dna_img_spot)) - 0.5;
        cc = corr2(prot_img,dna_img); % some background good for normalisation

        k;
        prot_img;

        % if k == 17
        %     figure(1);
        %     imshow(prot_img,[min(min(prot_img)),max(max((prot_img)))]);
        %     figure(2);
        %     imshow(dna_img,[min(min(dna_img)),max(max((dna_img)))]);
        % end
        
        % output
        foo = strsplit(deetfile,'_fitdetail');
        colofile = [foo{1},'_colocal_first.csv'];
        f = fopen(colofile,'a');
        fprintf(f,'%d,%d,%d,%.2f,%.2f,%.2f,%.2f,%.2f\n',[pifparam(1:3),prot_x,prot_y,dna_x,dna_y,cc]);
        fclose(f);

    end

end

