% Evaluate fit with x number of steps from ranked list

function steplist = evalsteps(intfile,x,y,ns_custom)
        
    min_duration = 5; % minumum step duration
    max_steps = 10; % number of steps to fit
    frame_end = 1500; % throw away useless frames at tail end of movie

    if (isstr(intfile) == 0) || (exist(intfile) == 0) % short circuit; second condition requires string
        intfile = uigetfile('*.csv');
    end
    t = strsplit(intfile,'/')   ;             
    t = strsplit(t{end},'_intensities.csv');
    rootname = t{1};

    M = csvread(intfile,1,0);
    dna_coord = M(:,1:2);
    trace_ind = find(ismember(dna_coord,[x,y],'rows'));
    if trace_ind == []
        disp('Coordinates not found.');
    else
        raw = M(trace_ind,3:end);
    end
    filtered = ckfilter(raw,[10 15 20],8,5); % apply CK filter
    grad = diff(filtered);
    [foo, frame_zero] = max(grad); % sharpest rise; laser on
    steplist = findsteps(filtered,min_duration,max_steps,frame_zero,frame_end);

    % plot input 
    figure(gcf);
    plot(raw(1:frame_end),'LineWidth',1.5);
    hold on;
    plot(filtered(1:frame_end),'LineWidth',1.5);
    hold off;
    
    if steplist(ns_custom,3)/steplist(ns_custom,2) < 1
        disp('Warning: counter-fit better than fit.');
    end
    
    % calculate step levels (y)
    seg_points = sort([frame_zero; steplist(1:ns_custom,1); frame_end]);
    mean_fit = zeros(ns_custom+1,1);
    for j = 1:ns_custom+1
        seg = filtered(seg_points(j)+1:seg_points(j+1));
        mean_fit(j) = sqrt(sum(seg.^2)/length(seg));
    end

    xfit = reshape([transpose(seg_points);transpose(seg_points+1)],[],1);
    xfit = xfit(2:end-1); % generate x values to plot fit for number of steps = ns_custom
    yfit = reshape(repmat(transpose(mean_fit),2,1),[],1);
    hold on;
    plot(xfit,yfit,'LineWidth',1.5);
    hold off;

end


% two-sample t-test, unequal variance
% N=10;
% alpha=0.05;
% x1 = movmean(test,[0 N-1],'Endpoints','fill'); % moving mean, current and next N-1 frames, NaN if elements missing in window
% x2 = circshift(x1,N,2); % moving mean, previous N frames
% x2(1) = NaN; % correct first element
% s21 = movvar(test,[0 N-1],'Endpoints','fill'); % moving variance, s squared 1, current and next N-1 frames
% s22 = circshift(s1,N,2); % moving variance, previous N frames
% s22(1) = NaN;
% t = abs((x1-x2)./sqrt((x1.^2+x2.^2)/N));
% df = ((s21+s22)/N)./((s21.^2+s22.^2)/(N*(N-1)));
% tcrit = tinv(1-alpha/2,df);
% stepidx = find(t > tcrit);
