%% Fit 2D gaussian, with integration over pixel area for calculation of residuals
function fitresult = fit_2D_gaussian(img)

    % Prepare data for fitting
    [x,y,z] = prepareSurfaceData ( 1:size(img,2) , 1:size(img,1) , img ); % generate column vectors of matching x,y coordinates and intensity values; top left pixel is (1,1)
    xy = {x,y};

    % Starting centroid position, gaussian widths, baseline, etc.
    [A, ind] = max(z); % define height, return index for intensity maximum
    theta1 = 0;
    x1 = x(ind);
    y1 = y(ind);
    s_x1 = 1;
    s_y1 = 1;
    C = median(z)-std(z);
    par = [A, theta1, x1, y1, s_x1, s_y1, C];

    % Set bounds
    lowr = [0, 0, 0, 0, 0, 0, 0];
    uppr = [Inf, 2*pi, Inf, Inf, max(x), max(y), Inf];

    % Set options
    options = optimoptions('lsqcurvefit','Display','off',...
                                         'MaxFunEvals',5e2,...
                                         'MaxIter',5e2,...
                                         'TolX',1e-16,...
                                         'TolFun',1e-16);

    % Perform fit
    [fitresult,resnorm,residual,exitflag,output,lambda,jacobian] = lsqcurvefit(@z_hat_discrete,par,xy,z,lowr,uppr,options);
    
    % err = sqrt(diag(inv(jacobian'*jacobian)*resnorm/(spot_size^2-size(par,2))));
    % err = full(transpose(err));
    
end

%% Integrate gaussian over pixels for sqcurvefit
function z = z_hat_discrete(par,xy)

    z = zeros(length(xy{1}),1);
    
    for k = 1:length(z)
        xmax = xy{1}(k);
        xmin = xmax-1;
        ymax = xy{2}(k);
        ymin = ymax-1;
        z(k) = integral2( @(x,y) gaussian_2D(par,{x,y}) , xmin, xmax, ymin, ymax);
    end

end
