%% Fit symmetric 2D gaussian, with integration over pixel area for calculation of residuals, taking circle of radius rad and origin in the center
function fitresult = fit_2D_gaussian_sym_circle(img, rad)

    % Prepare data for fitting
    [x,y,z] = prepareSurfaceData ( 1:size(img,2) , 1:size(img,1) , img ); % generate column vectors of matching x,y coordinates and intensity values; top left pixel is (1,1)

    centre_x = (size(img,2)+1)/2;
    centre_y = (size(img,1)+1)/2;
    [xgrid,ygrid] = meshgrid(1:size(img,2),1:size(img,1));
    spot_mask = sqrt((xgrid-centre_x).^2+(ygrid-centre_y).^2)<=rad; % generate mask
    spot_px = sum(sum(spot_mask));

    x = x(spot_mask(:)); % reduce list to pixels in mask
    y = y(spot_mask(:));
    z = z(spot_mask(:));

    xy = {x,y};

    % Starting centroid position, gaussian widths, baseline, etc.
    [A0, ind] = max(z); % define height, return index for intensity maximum
    x0 = x(ind);
    y0 = y(ind);
    s0 = 2;
    C0 = median(z)-std(z);
    par0 = [A0, x0, y0, s0, C0];

    % Set bounds
    lowr = [0, 0, 0, 0, 0];
    uppr = [Inf, max(x), max(y), max(x), max(z)];

    % Set options
    options = optimoptions('lsqcurvefit','Display','off',...
                                         'MaxFunEvals',5e2,...
                                         'MaxIter',5e2,...
                                         'TolX',1e-16,...
                                         'TolFun',1e-16);

    % Perform fit
    [fitresult,resnorm,residual,exitflag,output,lambda,jacobian] = lsqcurvefit(@z_hat_discrete,par0,xy,z,lowr,uppr,options);
    
    % err = sqrt(diag(inv(jacobian'*jacobian)*resnorm/(spot_size^2-length(par))));
    % err = full(transpose(err));
    
end

%% Integrate gaussian over pixels for sqcurvefit
function z = z_hat_discrete(par,xy)

    z = zeros(length(xy{1}),1);
    
    for k = 1:length(z)
        xmax = xy{1}(k);
        xmin = xmax-1;
        ymax = xy{2}(k);
        ymin = ymax-1;
        z(k) = integral2( @(x,y) gaussian_2D_sym(par,{x,y}) , xmin, xmax, ymin, ymax);
    end

end
