% PIF (unfiltered trace)

function nsteps = pif(obs)

    format shortG

    frame_end = 1500; % throw away useless frames at tail end of movie
    obs = obs(1:frame_end);

    ckparam = [8,5;5,3]; % two filter parameter sets
    grad = diff(ckfilter(obs,[10 15 20],ckparam(1,1),ckparam(1,2)));
    [foo, frame_zero] = max(grad); % sharpest rise; laser on

    for k = 1:2
        lvls = []; % reset
        ratio_ref = 1; % reset fit ratio criterion
        chisq_ref = Inf; % reset chisq criterion
        filtered = ckfilter(obs,[10 15 20],ckparam(k,1),ckparam(k,2)); % apply CK filter
        for SNR = [0.1, 0.25, 0.5, 1]
            [lvls_new,chisq_fit_new,chisq_counter_new] = eval_reval(filtered,frame_zero,frame_end,SNR);
            ratio_new = chisq_counter_new/chisq_fit_new; 
            if  ratio_new > ratio_ref & chisq_fit_new/chisq_ref <  1.5 % better ratio, and either decresases chisq_fit or increases no more than half 
            % if chisq_fit_new < chisq_ref & chisq_counter_new/chisq_fit_new > 1 % chisq criterion
                lvls = lvls_new;
                chisq_fit = chisq_fit_new;
                chisq_counter = chisq_counter_new;
                ratio_ref = chisq_counter/chisq_fit; % fit ratio criterion 
                chisq_ref = chisq_fit; % chisq criterion
            end
        end
        figure(k);
        plot(obs(1:frame_end),'Linewidth',0.5); % plot raw trace
        hold on;
        plot(ckfilter(obs,[10 15 20],ckparam(k,1),ckparam(k,2)),'LineWidth',1); % plot filtered trace
        hold off;
        disp(['Filter ',num2str(k)]);
        if isempty(lvls) == 0 % plot fit
            hold on;
            xfit = [frame_zero; reshape([transpose(lvls(:,1));transpose(lvls(:,1)+1)],[],1)];
            xfit = xfit(1:end-1);
            yfit = reshape(repmat(transpose(lvls(:,2)),2,1),[],1);
            plot(xfit,yfit,'LineWidth',1.5);
            hold off;
            disp('   last_frame     mean_val');
            disp(lvls);
            nsteps{k} = size(lvls,1)-1;
        else
            disp('No steps found. Counter-fit always better than fit...');
            disp(' ');
            nsteps{k} = 0;
        end
        axis([-Inf Inf -Inf max(ckfilter(obs,[10 15 20],ckparam(1,1),ckparam(1,2)))*1.2]);
        title(['filter ',num2str(k),'  ',num2str(max(size(lvls,1)-1,0)),' steps fitted']);
    end

end

function [lvls,chisq_fit,chisq_counter] = eval_reval(obs,frame_zero,frame_end,SNR)

    % Generate lvls list; column 1, idealised value; column 2, last frmae index
    lvls = transpose([(frame_zero+1:frame_end);obs(frame_zero+1:frame_end)]); % obs, row vector

    % Evaluation and Re-evaluation
    nlvls = size(lvls,1);
    nlvls_new = 1;
    while nlvls_new < nlvls
        mean_I_FF = (lvls(end,2)-lvls(1,2))/(lvls(end,1)-lvls(1,1));
        N_FF = sqrt((sum((diff(lvls(:,2))-mean_I_FF).^2))/(lvls(end,1)-lvls(1,1)));
        psy = 1.067*exp(-(SNR-0.999)/0.335)+1.824;
        sig = SNR * N_FF / psy;
        nlvls = size(lvls,1);
        lvls = anneal(lvls,frame_zero,sig);
        nlvls_new = size(lvls,1);
        SNR = mean(abs(diff(lvls(:,2))))/N_FF;
    end
    lvls = lvls(logical([1;diff(lvls(:,1))>2]),:); % discarding second of frames more than 2 apart
    nlvls = size(lvls,1);

    % Chi-squared value of fit
    seg_points = sort([frame_zero; lvls(:,1)]); % add new step to list of points
    chisq_fit = 0;
    for k = 1:nlvls
        seg = obs(seg_points(k)+1:seg_points(k+1));
        chisq_fit = chisq_fit + sum(((seg-lvls(k,2)).^2))/lvls(k,2);
    end
    
    % Chi-squared value of counter-fit
    seg_points = round(movmean(seg_points,[0 1],'Endpoints','discard'));
    seg_points = sort([frame_zero; seg_points; frame_end]); % add new step to list of points
    chisq_counter = 0;
    for k = 1:nlvls+1
        seg = obs(seg_points(k)+1:seg_points(k+1));
        chisq_counter = chisq_counter + sum(((seg-mean(seg)).^2))/mean(seg);
    end
    
end

% Anneal, local function
function lvls = anneal(lvls,frame_zero,sig)
    nlvls = size(lvls,1);
    nlvls_new = 1;
    for fac = flip(1:1000)
        thrs = sig/fac;
        nlvls = size(lvls,1);
        lvls = scan(lvls,frame_zero,thrs);
        nlvls_new = size(lvls,1);
    end
end


% Scan, local function
function lvls = scan(lvls,frame_zero,thrs)
    nlvls = size(lvls,1);
    nlvls_new = 1; % initialise
    while nlvls_new < nlvls
        nlvls = size(lvls,1);
        nlvls_new = 1;
        I_ref = lvls(1,2);
        lvl_length = lvls(1,1)-frame_zero; % frame count for current level
        lvls_new = zeros(size(lvls));
        k = 1;
        while k < nlvls
            % if I_ref-lvls(k+1,2)>thrs | I_ref-lvls(k+1,2)<0 % test next frame
            % if abs(I_ref-lvls(k+1,2))>thrs % test next frame;
            if I_ref-lvls(k+1,2)>thrs
                lvls_new(nlvls_new,1) = lvls(k,1);
                lvls_new(nlvls_new,2) = I_ref;
                nlvls_new = nlvls_new+1;
                I_ref = lvls(k+1,2);
                lvl_length = lvls(k+1,1)-lvls(k,1); % frame count of next level
            else
                I_ref = (I_ref*lvl_length+lvls(k+1,2)*(lvls(k+1,1)-lvls(k,1)))/(lvl_length + lvls(k+1,1)-lvls(k,1)); % include next level in mean
                lvl_length = lvl_length + lvls(k+1,1)-lvls(k,1);
            end
            k = k+1;
        end
        lvls = lvls_new(any(lvls_new,2),:);
        if isempty(lvls) == 1
            lvls = [frame_zero+lvl_length,I_ref];
        else
            lvls = [ lvls; [lvls(end,1)+lvl_length,I_ref] ];
        end
        
    end
end