% 161003 Evolve, TetraSpeck calibration
% upper left pixel in green channel = (5,17)
% bottom right pixel in green channel = (228,505)

% 161129 Evolve Delta, TetraSpeck calibration
% upper left pixel in green channel = (10,2)
% bottom right pixel in green channel = (243,510)

% 161201 Evolve, TetraSpeck calibration  <<< current
% upper left pixel in green channel = (12,5)
% upper left pixel in red channel = (276,6)

function pif_redo_smallcircle()

    % upper left corner of green and red channels
    green_x = 12-1;
    green_y = 5-1;
    % red_x = 276-1;
    % red_y = 6-1;

    % input and output
    rootname = 'large_small';
    intfile = [rootname,'_smallcircle_intensities.csv'];
    f = fopen(intfile,'w');
    fprintf(f,'Intensities\n');
    fprintf(f,'\n');
    fclose(f);
    warning('off','MATLAB:imagesci:tiffmexutils:libtiffWarning');

    % read summary
    fitsummary = csvread('161201_large_small_pif_custom_fitsummary.csv');

    for k = 1:size(fitsummary,1)

        pifparam = fitsummary(k,:);

        filename = [rootname,'_',num2str(pifparam(1)),'_MMImages.ome.tif'];
        % read tiff stack using LibTIFF routines (alternative to imread)
        info = imfinfo(filename);
        nframes = numel(info);
        tiff_obj = Tiff(filename,'r');
        tiff_stack = zeros(512,512,nframes);
        for k = 1:nframes
            tiff_obj.setDirectory(k);
            tiff_stack(:,:,k) = tiff_obj.read();
        end
        tiff_obj.close();

        % extract protein and dna tiff stacks
        xcoord = pifparam(2);
        ycoord = pifparam(3);
        prot_stack = tiff_stack((green_y+ycoord-3):(green_y+ycoord+3),(green_x+xcoord-3):(green_x+xcoord+3),1:1500);
        % dna_stack = tiff_stack((red_y+ycoord-8):(red_y+ycoord+8),(red_x+xcoord-8):(red_x+xcoord+8),1:2000);

        % extract small circle and calculate mean spot intensity, frame by frame
        [xgrid,ygrid] = meshgrid(1:7);
        spot_mask = sqrt((xgrid-4).^2+(ygrid-4).^2)<=3;
        spot_px = sum(sum(spot_mask));

        bg_mask_stack = repmat(not(spot_mask),1,1,size(prot_stack,3));
        prot_stack(bg_mask_stack) = 0;
        prot_intensities = transpose(squeeze(sum(sum(prot_stack,1),2)/spot_px));

        % pif based on filter in summary
        foo = pifparam(5:find(pifparam,1,'last'));
        % ckparam = {foo(1:end-2),foo(end-1),foo(end)};
        ckparam = {[10],5,5};
        pif_custom([xcoord,ycoord],prot_intensities,[rootname,'_',num2str(pifparam(1))],ckparam,'save all');

        % output
        f = fopen(intfile,'a');
        fprintf(f,strcat(num2str(pifparam(1)),',',num2str(xcoord),',',num2str(ycoord),',')); % write coordinates in first two columns
        fclose(f);
        dlmwrite(intfile,prot_intensities,'-append'); % write intensities (row vector) from third column

        % fitresult = fit_2D_gaussian_sym_zerobg(img_corrected);
        % xfit = fitresult(2)
        % yfit = fitresult(3)

    end

end