function pif_wrapper(varargin)

    frame_end = 1500;

    switch nargin
        case 3
            dna_coord = varargin{1};
            intensities = varargin{2};
            rootname  = varargin{3};
        otherwise
            if (nargin==1) & (isstr(varargin{1}) == 1) & (exist(varargin{1}) == 2)
                intfile = varargin{1};
            else
                intfile = uigetfile('*.csv');
            end
            M = csvread(intfile,1,0);
            dna_coord = M(:,1:2);
            for k = 1:size(M,1)
                intensities{k} = M(k,3:end);
            end
            t = strsplit(intfile,'/')   ;             
            t = strsplit(t{end},'_intensities.csv');
            rootname = t{1};
    end

    % write destinations
    if isempty(strfind(pwd,rootname))==0
        epsfile = strcat(rootname,'_pif_intensities.eps');
        fitfile = strcat(rootname,'_pif_fit.csv');
        fitlog = strcat(rootname,'_pif_fit.log');
    else
        epsfile = strcat(rootname,'/',rootname,'_pif_intensities.eps');
        fitfile = strcat(rootname,'/',rootname,'_pif_fit.csv');
        fitlog = strcat(rootname,'/',rootname,'_pif_fit.log');
        if exist(rootname) ~= 7
            mkdir(rootname);
        end
    end

    roottitle = strrep(rootname,'_','\_');

    diary(fitlog); % log terminal output
    close all;
    fig1 = figure(1);
    fig2 = figure(2);
    for k = 1:length(intensities)
        disp([num2str(dna_coord(k,1)),',',num2str(dna_coord(k,2))]); % log
        disp(' ');
        raw = intensities{k};
        nsteps = pif(raw);
        figure(1);
        h = get(gca,'Title'); % grab and append to title
        title([roottitle,'  ',num2str(dna_coord(k,1)),',',num2str(dna_coord(k,2)),'  ',h.String]);
        figure(2);
        h = get(gca,'Title'); % grab and append to title
        title([roottitle,'  ',num2str(dna_coord(k,1)),',',num2str(dna_coord(k,2)),'  ',h.String]);
        print(fig1, '-dpsc', '-append', epsfile);
        print(fig2, '-dpsc', '-append', epsfile);
        dlmwrite(fitfile,[dna_coord(k,1), dna_coord(k,2), 1, nsteps{1}], '-append');
        dlmwrite(fitfile,[dna_coord(k,1), dna_coord(k,2), 2, nsteps{2}], '-append');
        disp(' ');
    end
    close(fig1);
    close(fig2);
    diary off;
