% Wrapper for findsteps.m
% pass in data or read csv file (specified or dialog-selected)
% csv file, first line skipped, DNA coordinates in first two columns, intensities in remaining columns
% exports traces and fits as multi-page postscript

function stepfit_wrapper(varargin)

    frame_end = 1500;

    switch nargin
        case 3
            dna_coord = varargin{1};
            intensities = varargin{2};
            rootname  = varargin{3};
        otherwise
            if (nargin==1) & (isstr(varargin{1}) == 1) & (exist(varargin{1}) == 2)
                intfile = varargin{1};
            else
                intfile = uigetfile('*.csv');
            end
            M = csvread(intfile,1,0);
            dna_coord = M(:,1:2);
            for k = 1:size(M,1)
                intensities{k} = M(k,3:end);
            end
            t = strsplit(intfile,'/')   ;             
            t = strsplit(t{end},'_intensities.csv');
            rootname = t{1};
    end

    % write destinations
    if isempty(strfind(pwd,rootname))==0
        epsfile = strcat(rootname,'_chi_intensities.eps');
        fitfile = strcat(rootname,'_chi_fit.csv');
        fitlog = strcat(rootname,'_chi_fit.log');
    else
        epsfile = strcat(rootname,'/',rootname,'_chi_intensities.eps');
        fitfile = strcat(rootname,'/',rootname,'_chi_fit.csv');
        fitlog = strcat(rootname,'/',rootname,'_chi_fit.log');
        if exist(rootname) ~= 7
            mkdir(rootname);
        end
    end

    diary(fitlog); % log terminal output
    fig = figure();
    for k = 1:length(intensities)
        disp([num2str(dna_coord(k,1)),',',num2str(dna_coord(k,2))]); % log
        raw = intensities{k};
        plot(raw(1:frame_end),'Linewidth',0.5);
        filtered = ckfilter(raw,[10 15 20],8,5); % apply CK filter
        [foo,ns_rec] = autosteps(filtered);
        title([num2str(dna_coord(k,1)),',',num2str(dna_coord(k,2)),' ',num2str(ns_rec),' steps fitted']);
        if isempty(foo) == 0
            disp('Steplist:');
            disp('     step_ind    fit_chisq   cntr_chisq  magic_ratio');
            disp(foo);
        else
            disp(' ');
        end
        print(fig, '-dpsc', '-append', epsfile);
        dlmwrite(fitfile,[dna_coord(k,1), dna_coord(k,2), ns_rec],'-append');
    end
    close(fig);
    diary off;

end